(function(){
    function extend(){
        var options, name, src, copy, copyIsArray, clone,
            target = arguments[0] || {},
            i = 1,
            length = arguments.length,
            deep = false;

        // Handle a deep copy situation
        if (typeof target === 'boolean') {
            deep = target;
            target = arguments[1] || {};
            // skip the boolean and the target
            i = 2;
        }

        // Handle case when target is a string or something (possible in deep copy)
        if (typeof target !== 'object' && type(target) !== 'function') {
            target = {};
        }

        // extend caller itself if only one argument is passed
        if (length === i) {
            target = this;
            --i;
        }

        for (; i<length; i++) {
            // Only deal with non-null/undefined values
            if ((options = arguments[i]) != null) {
                // Extend the base object
                for (name in options) {
                    src = target[name];
                    copy = options[name];

                    // Prevent never-ending loop
                    if (target === copy) {
                        continue;
                    }

                    // Recurse if we're merging plain objects or arrays
                    if (deep && copy && (isPlainObject(copy) || (copyIsArray = type(copy) === 'array'))) {
                        if (copyIsArray) {
                            copyIsArray = false;
                            clone = src && type(src) === 'array' ? src : [];
                        } else {
                            clone = src && isPlainObject(src) ? src : {};
                        }

                        // Never move original objects, clone them
                        target[name] = extend(deep, clone, copy);

                        // Don't bring in undefined values
                    } else if (copy !== undefined) {
                        target[name] = copy;
                    }
                }
            }
        }

        // Return the modified object
        return target;

    }

    function param (data, appendTo) {
        var stack = [],
            query;

        for(key in data) {
            stack.push(encodeURIComponent(key) + '=' + encodeURIComponent(data[key]));
        }

        query = stack.join('&').replace(/%20/g, '+');

        if (typeof appendTo === 'string') {
            query = appendTo + (query.length > 0 ?
                (appendTo.indexOf('?') < 0 ? '?' : '&') + query : '');
        }

        return query;
    }

    function ping(url, params){
        var img = new Image();
        params = extend({
            _tm: new Date() - 0
        }, params);
        img.src = param(params, url);
    }

    function setCookie(key, val, time){//设置cookie方法
        var date=new Date(); //获取当前时间
        var expiresDays = time;  //将date设置为n天以后的时间
        date.setTime(date.getTime()+expiresDays*24*3600*1000); //格式化为cookie识别的时间
        document.cookie=key + "=" + val +";expires="+date.toGMTString();  //设置cookie
    }

    function getCookie (key){//获取cookie方法
        /*获取cookie参数*/
        var getCookie = document.cookie.replace(/[ ]/g,"");  //获取cookie，并且将获得的cookie格式化，去掉空格字符
        var arrCookie = getCookie.split(";")  //将获得的cookie以"分号"为标识 将cookie保存到arrCookie的数组中
        var tips;  //声明变量tips
        for(var i=0;i<arrCookie.length;i++){   //使用for循环查找cookie中的tips变量
            var arr=arrCookie[i].split("=");   //将单条cookie用"等号"为标识，将单条cookie保存为arr数组
            if(key==arr[0]){  //匹配变量名称，其中arr[0]是指的cookie名称，如果该条变量为tips则执行判断语句中的赋值操作
                tips=arr[1];   //将cookie的值赋给变量tips
                break;   //终止for循环遍历
            }
        }
        return tips;
    }

    /**
     * 生成唯一标识码的方法
     * @returns {string}
     */
    function uuid() {
        function s4() {
            return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
        }
        return s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();
    }

    /**
     * 格式化js统计的信息
     * 格式为：
     * @param statParams {object} 统计参数对象 eg:{page:'index', tag: '热门'}
     * @returns {string} 格式化后的统计信息 'page=index&tag=热门'
     */
    function formatStatParams (statParams) {
        var keys = Object.keys(statParams),
            plist = [];

        keys.forEach(function(key){
            plist.push(key + '=' + statParams[key]);
        });

        return plist.join('&');
    }

    function WaStat(){
        var that = this;

        //WA统计的URL
        that.statUrl = 'http://track.uc.cn:9080/collect?uc_param_str=dnfrcpve&';
        //that.statUrl = 'http://dev7.ucweb.local:8888/collect?uc_param_str=dnfrcpve&';
        //统计用到的公共配置
        that.comStat = {};

        //重写构造函数实现单例
        WaStat = function() {
            return that;
        }
    }


    /**
     * 设置 wa 统计公共参数的方法
     * @param olpmParams olpm系统返回的UC公参信息或者自定义公参信息
     * eg: {appid:xxx, pg: xxx, ch:xxx}
     *
     */
    WaStat.prototype.setComStat = function(olpmParams) {
        var that = this;

        that.comStat = extend(that.comStat, olpmParams);

        //默认给WA统计添加 uuid 方便在非UC浏览器下面作UV统计
        if(!that.comStat.uuid) {
            that.comStat.uuid = that.getUnUCUid();
        }
    };

    /**
     * 获取唯一码的方法
     * @returns {*}
     */
    WaStat.prototype.getUnUCUid = function () {
        var key = location.hostname + '_uid',
            uvId = '';

        try {
            if(!localStorage.getItem(key)) {
                localStorage.setItem(key, uuid());
            }
            uvId = localStorage.getItem(key);
        } catch (e) {
            if(!getCookie(key)) {
                setCookie(key, uuid(), 360);
            }
            uvId = getCookie(key);
        }

        return uvId;
    };

    /**
     * WA pv 日志统计的方法
     * @param option 自定义参数
     */
    WaStat.prototype.pvStat = function(option) {
        var that = this;

        if(!option) {
            option = {};
        }
        //pv统计的类型标识
        option.lt = 'log';
        option.pv_rf = document.referrer;

        var paramStr = formatStatParams(extend(option, that.comStat));
        // console.log(that.statUrl + paramStr);
        ping(that.statUrl + paramStr);
    };

    /**
     * WA click 日志统计的方法
     * @param option 自定义参数
     */
    WaStat.prototype.clickStat = function(option) {
        var that = this;

        if(!option) {
            option = {};
        }
        //click统计的类型标识
        option.lt = 'link';

        var paramStr = formatStatParams(extend(option, that.comStat));
        // console.log(that.statUrl + paramStr);
        ping(that.statUrl + paramStr);
    };

    /**
     * WA event 日志统计的方法
     * @param option 自定义参数
     */
    WaStat.prototype.eventStat = function(option) {
        var that = this;

        if(!option) {
            option = {};
        }
        //event统计的类型标识
        option.lt = 'event';
        option.e_c = '行为统计';
        option.e_a = 'touch';

        var paramStr = formatStatParams(extend(option, that.comStat));
        // console.log(that.statUrl + paramStr);
        ping(that.statUrl + paramStr);
    };

    WaStat.prototype.getUrlParam= function(name, url){

        var reg = new RegExp('(^|&)'+ name + '=([^&]*)(&|$)'), r;

        if(url) {
            r = url.substr(url.indexOf('?') + 1).match(reg);
        } else {
            r = window.location.search.substr(1).match(reg);
        }

        //console.log(r);

        return r !== null ? unescape(r[2]) : null;
    };

    if(!window.waStat) {
        window.waStat = new WaStat();
    }

    //使用事件委托来实现特定站点添加wa的click统计
    document.addEventListener('click', function(e){
        e.preventDefault();
        var target = e.target;

        while(target !== e.currentTarget) {
            try{
                //如果以ext开头的站点且带有属性data-wa=true，则该站点要作wa的click点击统计
                if(target.tagName.toLowerCase() === 'a'|| target.tagName.toLowerCase() === 'li'&& typeof target.getAttribute('href') != 'undefined') {//&& target.dataset.wa === 'true'
                    //console.log(target.tagName);

                    var siteUrl = target.getAttribute('href'),
                        statInfo = {};

                    if(/^ext:/.test(siteUrl)) {
                        //ext:as:navi_online_index-479*598*1907-N33416:http://3g.163.com/ntes/special/0034073A/wechat_article.html?docid=A3H0ICBG00963VRO
                        //ext:as:navi_online_pages-529*1327-N6913:http://www.digu.com/
                        var statStr = siteUrl.substring(0, siteUrl.indexOf(':http')),
                            statList = statStr.split('-'),
                            statIds = statList[1].split('*');

                        statInfo = {
                            appid:'040e7c1d4ff0',
                            pg:'huawei',
                            fcid: statIds[0]||'',
                            cid: statIds[1]||'',
                            pox: statIds.length === 3 ? statIds[2] : '',
                            uid: statList[2].substring(0, 1).toUpperCase() === 'U' ? statList[2].substring(1) : '',
                            nid: statList[2].substring(0, 1).toUpperCase() === 'N' ? statList[2].substring(1) : '',
                            title: target.title
                        };

                        window.waStat.clickStat(statInfo);

                        sendElementUrlOnLongPress(target,2);
                    }else if (window.waStat.getUrlParam('fcid', siteUrl)) {
                        //http://stat.hao.uc.cn/stat_site.php?nid=41030&fcid=2031&cid=2024&page=navi_online_index
                        //http://stat.hao.uc.cn/stat_site.php?uid=1030&fcid=2031&cid=2024&page=navi_online_index
                        statInfo = {
                            appid:'040e7c1d4ff0',
                            pg:'huawei',
                            fcid: window.waStat.getUrlParam('fcid', siteUrl) || '',
                            cid: window.waStat.getUrlParam('cid', siteUrl) || '',
                            nid: window.waStat.getUrlParam('nid', siteUrl) || '',
                            uid: window.waStat.getUrlParam('uid', siteUrl) || '',
                            title: target.title
                        };
                        window.waStat.clickStat(statInfo);

                        sendElementUrlOnLongPress(target,2);
                    } else {
                        statInfo = {
                            url : siteUrl,
                            title: target.title || ''
                        };
                    }

                    //调用wa 的click统计
                    //console.log(JSON.stringify(statInfo));

                    //站点跳转
                    // setTimeout(function() {//console.log(siteUrl);
                    //     location.href = siteUrl;
                    // },200);
                }

                target = target.parentNode;
            } catch(e){
                console.error(e.message);
                return;
            }
        }


    }, false);

    if ( typeof module != 'undefined' && module.exports ) {
        module.exports = window.waStat;
    }
})();
