﻿/******************************************************************
 * @文件名称 : reader.js
 * @文件作者 : l00191734 yan.liu
 * @创建时间 : 2012-09-04
 * @文件描述 : 阅读器js：解析网页
 * @版权声明 : Copyright (C) 2012-2014 华为技术有限公司(Huawei Tech.Co.,Ltd)
 * @修改历史 : 2012-09-04 1.00 初始版本
 ******************************************************************/
 
 /* 测试log开关 */
var debugMode = false;
var dbg = function(s) {	
	if (!debugMode){
		return;
	}	
	if(typeof console !== 'undefined')
		 console.log("\n " + s); 
};
 
var readability = {
	iframeLoads: 0,	
	bodyCache:  null, 	// 缓存<body> 
	readerCache:null,   // 阅读器数据 
	readerHtml: null,	// 抽取网页的<body>作为用于阅读模式解析的新建标签数据
	isReadable: true,   // 是否可进入阅读模式
	tabPosition:-1,     // tab的标识
	
	
	// 正则表达式 
	regexps: {
		unlikelyCandidatesRe:   /combx|comment|community|disqus|extra|foot|header|menu|remark|meta|nav|rss|shoutbox|sidebar|sponsor|ad-break|agegate|pagination|pager|popup|tweet|twitter|advert|operation|list|user|player|scroll|tab|gallery|hide|descr|intro|appinfo|css/i, //不是正文的tag
		okMaybeItsACandidateRe: /and|article|body|column|main/i,
		positiveRe:             /article|body|content|entry|hentry|page|pagination|post|text/i,
		negativeRe:             /combx|comment|contact|foot|footer|footnote|link|media|meta|promo|related|scroll|shoutbox|sponsor|tags|widget/i,
		divToPElementsRe:       /<(a|blockquote|dl|div|img|ol|p|pre|table|ul)/i,
		replaceBrsRe:           /(<br[^>]*>[ \n\r\t]*){2,}/gi,
		replaceFontsRe:         /<(\/?)font[^>]*>/gi,
		trimRe:                 /^\s+|\s+$/g,
		normalizeRe:            /\s{2,}/g,
		killBreaksRe:           /(<br\s*\/?>(\s|&nbsp;?)*){1,}/g,  
		videoRe:                /http:\/\/(www\.)?(youtube|vimeo)\.com/i,
		
		/* 标题匹配表达式 */
		_ArticleTitleDashRegex1: /[\\|\\-\\_]/g,
		_ArticleTitleDashRegex2: /(.*)[\\|\\-\\_].*/g,
		_ArticleTitleDashRegex3: /[^\\|\\-\\_]*[\\|\\-\\_](.*)/g,
		_ArticleTitleColonRegex1:  /.*:(.*)/g,
		_ArticleTitleColonRegex2: /[^:]*[:](.*)/g
	},

    /** 
     * 主函数：运行js
     * @return void 
     **/ 
	init: function(preserveUnlikelyCandidates) { 
		// 1. 初始化
		readability.isReadable = true;
		// 是否保留不太可能的节点 —— false:首先删除不太可能的节点； true:保留不删除
		preserveUnlikelyCandidates = (typeof preserveUnlikelyCandidates == 'undefined') ? false : preserveUnlikelyCandidates;		
		 
		if(document.body && !readability.bodyCache){
			readability.bodyCache = document.body.innerHTML; 
			readability.readerHtml = document.body.innerHTML; 
		}
		readability.prepFrameSet();	
		// 如果不存在阅读数据则终止解析
		if(readability.readerHtml==null) return;
		
		// 创建用于阅读模式解析的div标签
		var readerDiv = document.createElement("DIV"); 
		readerDiv.innerHTML = readability.readerHtml;
		 
		
		// 2. 准备，删除无用节点
		readability.prepDocument(readerDiv); 
		 
		// 3. 解析html，获取标题与正文
		var articleTitle   = readability.getArticleTitle();
		var articleContent = readability.grabArticle(preserveUnlikelyCandidates,readerDiv);		
		
		// 4. 条件判断是否可进入阅读模式 
		/**
		 * 解析获得正文为空时 
		 * if...如果首先删除了不太可能的节点，则保留不太可能的节点重新解析，防止正文被误删; 
		 * else...否则此网页无正文 ，不可进入阅读模式
		 **/ 
		if(readability.getInnerText(articleContent, false) == "")
		{
			//deleted for 用户评论很长时被解析为正文进入阅读模式
			/*if(!preserveUnlikelyCandidates) {
				document.body.innerHTML = readability.bodyCache;
				return readability.init(true);				
			} else {
				readability.isReadable = false; 	
				//articleContent.innerHTML = "<p>Sorry, this page was unable to be parsed for content... </p>";						
			}*/
			readability.isReadable = false; 	
		}
		
		// 字符数小于特定值不可进入阅读器 
		if(readability.isReadable){
			var str = readability.getInnerText(articleContent, false).replace(/\s{1,}/g, ""); //去除空格
//			dbg("文本: "+str+"\n字数：" + str.length); 	
			if(str.length < 300) {
				readability.isReadable = false; 
//				dbg("**********字数为  " + str.length + " 少于 300 个字符不可进入阅读模式！**********");
			}
		}
		readability.readerCache = "<p>Sorry, this page was unable to be parsed for content... </p>";
		// 5. 构造 DOM tree 
		if(readability.isReadable){
			var overlay        = document.createElement("DIV");
			var innerDiv       = document.createElement("DIV");		
			var hrTag       = document.createElement("hr");		
			overlay.id  = "readOverlay";
			innerDiv.id = "readInner";
	    	innerDiv.appendChild( articleTitle   );
	    	//间隔线
	    	innerDiv.appendChild( hrTag   );
	    	innerDiv.appendChild( articleContent );  	    
	    	overlay.appendChild( innerDiv       );
			 
			readability.readerCache = overlay.innerHTML;
		} 
		readerDiv.innerHTML = ""; 
		// 6. 将数据传入程序	
		readability.readerCache = readability.readerCache!=null ? readability.readerCache : "<p>Sorry, this page was unable to be parsed for content... </p>";
//		dbg("**********阅读器数据*************\n"+readability.readerCache);
		//dbg(window.jsInterface);
		if(window.jsInterface != null){
			window.jsInterface.htmlDataToObject(readability.readerCache,readability.isReadable, readability.tabPosition); 
		}
 		 	 	
	},

	/**
	 * 网页为框架结构时的处理函数
	 *
	 **/
	prepFrameSet: function () {
		 
		// 网页为框架结构时取绝对宽度与绝对高度之和最大的框架为主框架
		var frames = document.getElementsByTagName('frame');
		if(frames.length > 0)
		{
			var bestFrame = null;
			var bestFrameSize = 0;                                                                                                                 
			for(var frameIndex = 0; frameIndex < frames.length; frameIndex++)
			{
				var frameSize = frames[frameIndex].offsetWidth + frames[frameIndex].offsetHeight; //绝对宽度 + 绝对高度
				var canAccessFrame = false;
				try {
					frames[frameIndex].contentWindow.document.body;
					canAccessFrame = true;
				} catch(e) {}
				
				if(canAccessFrame && frameSize > bestFrameSize)
				{
					bestFrame = frames[frameIndex];
					bestFrameSize = frameSize;
				}
			}

			if(bestFrame)
			{ 
				readility.readerHtml = 	bestFrame.contentWindow.document.body.innerHTML;							
			}
		}
	},
	
	/**
     * Prepare the HTML document for readability to scrape it.
     * This includes things like stripping javascript, CSS, and handling terrible markup.
     * 
     * @return void
     **/
	prepDocument: function (e) {
		//dbg("=============prepDocument==============\n");
		
		 /* remove all scripts that are not readability */
		var scripts = e.getElementsByTagName('script');
		for(i = scripts.length-1; i >= 0; i--)
		{
			if(typeof(scripts[i].src) == "undefined" || scripts[i].src.indexOf('readability') == -1)
			{
				scripts[i].parentNode.removeChild(scripts[i]);			
			}
		}

		// 删除noscript
		readability.clean(e,"noscript"); 
		// 删除内联框架标签
		readability.clean(e,"iframe");
		// 删除嵌入标签
		readability.clean(e, "object");
		readability.clean(e, "embed");
		
		// 删除所有的样式  style 
		readability.clean(e,"style"); 
		
		// <section>结构不解析，如淘宝网宝贝介绍
		readability.clean(e, "section");
		
		// 删除隐藏节点
		readability.cleanHiddenNode(e);
		 
		// 替换掉连续2个以上的br标签为一个 -> 替换font标签(在最新的 HTML 版本（HTML 4 和 XHTML）中，字体标签已被废弃) 
		e.innerHTML = e.innerHTML.replace(readability.regexps.replaceBrsRe, '<br/>').replace(readability.regexps.replaceFontsRe, '<$1span>');
	},
	
	/**
	 * 获取标题
	 *
	 * @return void
	 **/
	getArticleTitle: function () {		
		var curTitle = "",
            origTitle = "";
        try {
            curTitle = origTitle = document.title;           
            if(typeof curTitle !== "string") {   // If they had an element with id "title" in their HTML  
                curTitle = origTitle = readability.getInnerText(document.getElementsByTagName('title')[0]);             
            }
        }
        catch(e) {}
        
        //主要针对英文字符
        if(curTitle.match(/ [\|\-] /))
        {
            curTitle = origTitle.replace(/(.*)[\|\-] .*/gi,'$1');            
            if(curTitle.split(' ').length < 3) {
                curTitle = origTitle.replace(/[^\|\-]*[\|\-](.*)/gi,'$1');
            }
        }
        else if(curTitle.indexOf(': ') !== -1)
        {
            curTitle = origTitle.replace(/.*:(.*)/gi, '$1');
            if(curTitle.split(' ').length < 3) {
                curTitle = origTitle.replace(/[^:]*[:](.*)/gi,'$1');
            }
        }
        else if(curTitle.length > 150 || curTitle.length < 15)
        {
            var hOnes = document.getElementsByTagName('h1');
            if(hOnes.length === 1)
            {
                curTitle = readability.getInnerText(hOnes[0]);
            }
        }

        curTitle = curTitle.replace( readability.regexps.trim, "" );
        
        if(curTitle.split(' ').length <= 4) {
            curTitle = origTitle;
        }
        //dbg("标题：  " + curTitle);
        var articleTitle = document.createElement("H1");
        articleTitle.innerHTML = curTitle; 
        return articleTitle;
	},	
	
 
	/***
     * grabArticle 
	 * Using a variety of metrics (content score, classname, element types), 
	 * find the content that is most likely to be the stuff a user wants to read. 
     * Then return it wrapped up in a div.
     *
     * @param page a document to run upon. Needs to be a full document, complete with body.
     * @return Element
    **/
	grabArticle: function (preserveUnlikelyCandidates,e) {
		/**
         * First, node prepping. 
         * Trash nodes that look cruddy (like ones with the class name "comment", etc), 
         * and turn divs into P tags where they have been used inappropriately (as in, where they contain no other block level elements.)
         *
         * Note: Assignment from index for performance. See http://www.peachpit.com/articles/article.aspx?p=31567&seqNum=5
         * TODO: Shouldn't this be a reverse traversal?
        **/
        //1. 准备：删除无用节点；将无子节点的div标签以及文本节点替换为p标签
		for(var nodeIndex = 0; (node = e.getElementsByTagName('*')[nodeIndex]); nodeIndex++)
		{
			// 删除无用节点
			if (!preserveUnlikelyCandidates) {
				var unlikelyMatchString = node.className + node.id;
				if (unlikelyMatchString.search(readability.regexps.unlikelyCandidatesRe) !== -1 &&
				    unlikelyMatchString.search(readability.regexps.okMaybeItsACandidateRe) == -1 &&
					node.tagName !== "BODY") {
					//dbg("Removing unlikely candidate ：  " + unlikelyMatchString);
					node.parentNode.removeChild(node);
					nodeIndex--;
					continue;
				}				
			}
			
			// 将没有子节点的 div 标签替换为 p 标签
			if (node.tagName === "DIV") {
				if (node.innerHTML.search(readability.regexps.divToPElementsRe) === -1)	{
					var newNode = document.createElement('p');
					try {
						newNode.innerHTML = node.innerHTML;				
						node.parentNode.replaceChild(newNode, node);
						nodeIndex--;
					}
					catch(e) {
						dbg("异常！！！\n 将没有子节点的 div 标签替换为 p 标签\n" + e);
					}
				}
				else
				{
					for(var i = 0, il = node.childNodes.length; i < il; i++) {
						var childNode = node.childNodes[i];
						if(childNode.nodeType == 3) {   // 文本
							//dbg("文本节点用p标签分段：\n" + childNode.nodeValue ); 
							var p = document.createElement('p');
							p.innerHTML = childNode.nodeValue;
							p.style.display = 'inline';
							p.className = 'readability-styled';
							childNode.parentNode.replaceChild(p, childNode);
						}
					}
				}
			} 
		}
		
		//dbg("替换之后的html:\n" + e.innerHTML);
		
		/**
         * Loop through all paragraphs, and assign a score to them based on how content-y they look.
         * Then add their score to their parent node.
         *
         * A score is determined by things like number of commas, class names, etc. Maybe eventually link density.
        **/
        // 2. 计算每个p标签的父节点以及祖父节点的score
		var allParagraphs = e.getElementsByTagName("p");
		var candidates    = [];

		for (var j=0; j < allParagraphs.length; j++) 
		{
			var parentNode      = allParagraphs[j].parentNode;
			var grandParentNode = parentNode.parentNode;
			var innerText       = readability.getInnerText(allParagraphs[j]);
			
			// If this paragraph is less than 25 characters, don't even count it.
			if(innerText.length < 25){
				//dbg("小于25字符不计算：  " + innerText);
				continue;
			}
				
			// Initialize readability data for the parent.
			if(typeof parentNode.readability == 'undefined')
			{
				// 节点增加readability对象属性，并初始化readability.contentScore(与节点类型、classname、id相关）
				readability.initializeNode(parentNode);
				candidates.push(parentNode);
			}
			// Initialize readability data for the grandparent. 
			if(grandParentNode && typeof grandParentNode.readability == 'undefined')
			{
				readability.initializeNode(grandParentNode);
				candidates.push(grandParentNode);
			}

			var contentScore = 0;
			
			// Add a point for the paragraph itself as a base.  
			contentScore++;
			
			// Add points for any commas within this paragraph 
			contentScore += innerText.split(',').length + innerText.split('，').length -1;
		 
			// For every 100 characters in this paragraph, add another point. Up to 3 points. 
			contentScore += Math.min(Math.floor(innerText.length / 100), 3);
			
			// Add the score to the parent. The grandparent gets half.  
			parentNode.readability.contentScore += contentScore;
			if(grandParentNode){
				grandParentNode.readability.contentScore += contentScore/2;
			}
		}

		// 3. 获取首选节点:最高分节点
		var topCandidate = null;
		for(var i=0, il=candidates.length; i < il; i++)
		{
			/**
             * Scale the final candidates score based on link density. 
             * Good content should have a relatively small link density (5% or less) and be mostly unaffected by this operation.
            **/
			candidates[i].readability.contentScore = candidates[i].readability.contentScore * (1-readability.getLinkDensity(candidates[i]));
 
			if(!topCandidate || candidates[i].readability.contentScore > topCandidate.readability.contentScore)
				topCandidate = candidates[i];
		}
		
		/**
         * If we still have no top candidate, just use the body as a last resort.
         * We also have to copy the body node so it is something we can modify.
         **/
		if (topCandidate == null || topCandidate.tagName == "BODY")
		{
			topCandidate = document.createElement("DIV");
			topCandidate.innerHTML = e.innerHTML;
			e.innerHTML = "";
			e.appendChild(topCandidate);
			readability.initializeNode(topCandidate);
		}

		// 4. 核查首选节点的兄弟姐妹节点
		/**
         * Now that we have the top candidate, look through its siblings for content that might also be related.
         * Things like preambles, content split by ads that we removed, etc.
        **/
        // 无兄弟姐妹则返回最高分节点
        if(topCandidate.parentNode == null){
       		readability.prepArticle(topCandidate);
        	return topCandidate;
        }       
		var articleContent        = document.createElement("DIV");
	    articleContent.id         = "readability-content";
	    // 门限值：分数不小于此值时节点保留
		var siblingScoreThreshold = Math.max(10, topCandidate.readability.contentScore * 0.2);
		var siblingNodes          = topCandidate.parentNode.childNodes;
		
		for(var i=0, il=siblingNodes.length; i < il; i++)
		{
			var siblingNode = siblingNodes[i];
			var append      = false; //节点是否保留
			
			if(!siblingNode) {
                continue;
            }
			
			//dbg("Looking at sibling node: " + siblingNode + " (" + siblingNode.className + ":" + siblingNode.id + ")" + ((typeof siblingNode.readability != 'undefined') ? (" with score " + siblingNode.readability.contentScore) : ''));
			//dbg("Sibling has score " + (siblingNode.readability ? siblingNode.readability.contentScore : 'Unknown'));

			if(siblingNode === topCandidate){
				append = true;
			}
			else {
				// 与首选节点同class时增分
				var contentBonus = 0;           	
           		if(siblingNode.className === topCandidate.className && topCandidate.className !== "") {
                	contentBonus += topCandidate.readability.contentScore * 0.2;
                	// 兄弟姐妹为div标签，且与首选节点相同class，不进入阅读模式，修复DTS2012101601444：屏蔽mmb.cn
                	if(siblingNode.nodeName == "DIV"){
                		//dbg("与最高分节点同class");
                		return "";
                	}
            	}
           		if(typeof siblingNode.readability !== 'undefined' && (siblingNode.readability.contentScore+contentBonus) >= siblingScoreThreshold) //r164
           		 // if(typeof siblingNode.readability != 'undefined' && siblingNode.readability.contentScore >= siblingScoreThreshold) //r107
				{
					append = true;
				}
			
				else if(siblingNode.nodeName == "P") {
					var linkDensity = readability.getLinkDensity(siblingNode);
					var nodeContent = readability.getInnerText(siblingNode);
					var nodeLength  = nodeContent.length;
				
					if(nodeLength > 80 && linkDensity < 0.25)
					{
						append = true;
					}
					else if(nodeLength < 80 && linkDensity == 0 && nodeContent.search(/\.( |$)/) !== -1)
					{
						append = true;
					}
				}
			}

			if(append)
			{ 
				//dbg("*************Appending node: " + siblingNode.nodeName + "\n" + siblingNode.innerHTML);

                var nodeToAppend = null;
                if(siblingNode.nodeName !== "DIV" && siblingNode.nodeName !== "P") {
                    /* We have a node that isn't a common block level element, like a form or td tag. Turn it into a div so it doesn't get filtered out later by accident. */                    
                    //dbg("Altering siblingNode of " + siblingNode.nodeName + ' to div.');
                    nodeToAppend = document.createElement("DIV");
                    try {
                        nodeToAppend.id = siblingNode.id;
                        nodeToAppend.innerHTML = siblingNode.innerHTML;
                    }
                    catch(er) {
                        //dbg("Could not alter siblingNode to div, probably an IE restriction, reverting back to original.");
                        nodeToAppend = siblingNode;
                        i-=1;
                        il-=1;
                    }
                } else {
                    nodeToAppend = siblingNode;
                    i-=1;
                    il-=1;
                }
                
                /* To ensure a node does not interfere with readability styles, remove its classnames */
                nodeToAppend.className = "";

                /* Append sibling and subtract from our list because it removes the node when you append to another node */
                articleContent.appendChild(nodeToAppend);
                //i--;
				//il--;
            }  
		}	
					
		// So we have all of the content that we need. Now we clean it up for presentation.
		readability.prepArticle(articleContent);
		
		return articleContent;
	},
	
	/**
     * Prepare the article node for display. 
     * Clean out any inline styles, iframes, forms, strip extraneous <p> tags, etc.
     *
     * @param Element
     * @return void
     **/
	prepArticle: function (articleContent) {
		// 1、删除junk
		readability.cleanStyles(articleContent); 
		//删除新浪网内嵌图片阅读器http://mil.news.sina.com.cn/2012-12-17/0952709880.html
		readability.clean(articleContent, "link");  
		//readability.killBreaks(articleContent); 
		readability.cleanHeaders(articleContent); 
		readability.clean(articleContent, "form");  
		//删除只出现一次的 <h1> - <h6> ，阅读器中的标题是<head>中的title标签		
		//If there is only one  <h1> - <h6> , they are probably using it as a header and not a subheader, so remove it since we already have a header.
        for(var i=0; i<=6; i++){
        	var hx = 'h' + i;
        	var hTags = articleContent.getElementsByTagName(hx);
        	if(hTags.length == 0)
        		continue;
        	else if(hTags.length == 1)
				readability.clean(articleContent, hx);	
			else
			{
				//替换为p标签，修复起点小说网结构不规范引起的字体变大（段落全部使用h类标签）
				for(var j=0 ; j<hTags.length ; j++){
					var newNode = document.createElement('p');
					try {
						//dbg("替换为p标签: "+hTags[j].innerHTML);
						newNode.innerHTML = hTags[j].innerHTML;				
						hTags[j].parentNode.replaceChild(newNode, hTags[j]);
					}
					catch(e) {
						dbg("" + e);
					}
				}
			}
        }
        readability.clean(articleContent, "dl");	
		readability.clean(articleContent, "ul");	
		readability.clean(articleContent, "ol");
		//删除多行的文本输入控件textarea，修复www.tudou.com误进入阅读模式
		readability.clean(articleContent, "textarea");
		//刪除单选或多选菜单
		readability.clean(articleContent, "select");  
		
		// Do these last as the previous stuff may have removed junk that will affect these
		//readability.cleanConditionally(articleContent, "table");
		readability.cleanConditionally(articleContent, "div");
		readability.clean(articleContent, "li");
		readability.clean(articleContent, "table");
		readability.clean(articleContent, "input");
		
		//正文中a标签密度大于0.1则不进入阅读模式
		//dbg("LinkDensity：" + readability.getLinkDensity(articleContent));
		if(readability.getLinkDensity(articleContent)>=0.1){
			articleContent.innerHTML = "" ;
			return ;
		}
		//删除大部分为a标签的p和div
		readability.cleanMostATag(articleContent, "p");
		readability.cleanMostATag(articleContent, "div");
		
	   //如果过滤后的正文存在相同class的div标签(如百度搜索），则不可进入阅读模式，do these last as 防止正文中可能存在相同class的广告div之前未过滤掉
       var divTags = articleContent.getElementsByTagName('div');
	   for(i = 0 ; i < divTags.length; i++){
	  	 if( divTags[i].className != "" ){
	  		 var j = i+1;
			 while( j < divTags.length ){
				if( divTags[i].className == divTags[j].className){
					articleContent.innerHTML = "" ;
					return ;
				}
				j++;
			}
		  }
	    }
	    
	    //删除空段落
		var articleParagraphs = articleContent.getElementsByTagName('p');
		for(i = articleParagraphs.length-1; i >= 0; i--)
		{
			var imgCount    = articleParagraphs[i].getElementsByTagName('img').length;
			//var embedCount  = articleParagraphs[i].getElementsByTagName('embed').length;
			//var objectCount = articleParagraphs[i].getElementsByTagName('object').length;
			
			//if(imgCount == 0 && embedCount == 0 && objectCount == 0 && readability.getInnerText(articleParagraphs[i], false) == '')
			if(imgCount == 0 && readability.getInnerText(articleParagraphs[i], false) == '')
			{
				articleParagraphs[i].parentNode.removeChild(articleParagraphs[i]);
			}
		}
		
		// 2、修改正文
		// <a>标签
		var aTagList = articleContent.getElementsByTagName('a');
		for(i = aTagList.length-1; i >= 0; i--)
		{
			if(aTagList[i].href == null) {
			}
			//删除javascript内嵌<a>标签
			else if(aTagList[i].href.indexOf("javascript:void(0)") != -1) {
				aTagList[i].parentNode.removeChild(aTagList[i]);
    		}
			//将<a>标签的相对链接转换为绝对链接
			else if(!aTagList[i].href.replace(readability.regexps.trimRe, "").substring(0,4)!="http") {
				aTagList[i].href = window.jsInterface.abstractUrl(readability.tabPosition,aTagList[i].href);
			}
		}
		
		// <img>标签
		var imgTagList = articleContent.getElementsByTagName('img');
		for(i = imgTagList.length-1; i >= 0; i--)
		{
			//img标签中使用假地址（自定义属性名）
			if(imgTagList[i].getAttribute("real_src") != null){
				imgTagList[i].src = imgTagList[i].getAttribute("real_src");
				imgTagList[i].removeAttribute("real_src");
			 }
			//将<img>标签的相对链接转换为绝对链接(如http://www.yznews.com.cn/2012-10/19/content_4103105.htm）
			if(!imgTagList[i].src.replace(readability.regexps.trimRe, "").substring(0,4)!="http"){
				imgTagList[i].src = window.jsInterface.abstractUrl(readability.tabPosition,imgTagList[i].src);
			}
		}
		
		try {
			//<br + 0个或多个非>号字符 + > + 0个或多个非空字符 + <p ——> <p
			//articleContent.innerHTML = articleContent.innerHTML.replace(/<br[^>]*>\s*<p/gi, '<p');		
		}
		catch (e) {
			dbg("line239"+"Cleaning innerHTML of breaks failed. This is an IE strict-block-elements bug. Ignoring.");
		}
	},
	
	/**
     * Initialize a node with the readability object. 
     * Also checks the className/id for special names to add to its score.
     *
     * @param Element
     * @return void
    **/
	initializeNode: function (node) {
		node.readability = {"contentScore": 0};			

		switch(node.tagName) {
			case 'DIV':
				node.readability.contentScore += 5;
				break;

			case 'PRE':
			case 'TD':
			case 'BLOCKQUOTE':
				node.readability.contentScore += 3;
				break;
				
			case 'ADDRESS':
			case 'OL':
			case 'UL':
			case 'DL':
			case 'DD':
			case 'DT':
			case 'LI':
			case 'FORM':
				node.readability.contentScore -= 3;
				break;

			case 'H1':
			case 'H2':
			case 'H3':
			case 'H4':
			case 'H5':
			case 'H6':
			case 'TH':
				node.readability.contentScore -= 5;
				break;
		}

		node.readability.contentScore += readability.getClassWeight(node);
	},
	
	 /**
     * 获取节点的文本
     * 
     * @param Element e:节点; normalizeSpaces:是否将两个连续的空白字符替换为一个，未定义时默认为false
     * @return string
    **/
	getInnerText: function (e, normalizeSpaces) {
		var textContent    = "";

		if(typeof(e.textContent) === "undefined" && typeof(e.innerText) === "undefined") {
            return "";
        }
        normalizeSpaces = (typeof normalizeSpaces == 'undefined') ? true : normalizeSpaces;
		textContent = e.textContent.replace( readability.regexps.trimRe, "" ); //去除首尾空白字符

		if(normalizeSpaces)
			return textContent.replace( readability.regexps.normalizeRe, " "); //将两个连续的空白字符替换为一个
		else
			return textContent;			
	},

	getCharCount: function (e,s) {
	    s = s || ",";
		return readability.getInnerText(e).split(s).length;
	},

	cleanStyles: function (e) {
	    e = e || document;
	    var cur = e.firstChild;
		if(!e)
			return;
		//令图片居中的style不删除 
		if(typeof e.removeAttribute == 'function' && e.className != 'readability-styled'&& e.getAttribute("style")!= "text-align:center;"){
			e.removeAttribute('style');
		}
	    while ( cur != null ) {
	    	var sibCur = cur.nextSibling;
			if ( cur.nodeType == 1 ) {
				    if(cur.getAttribute("style")!= "text-align:center;" && cur.className != "readability-styled") {
					    cur.removeAttribute("style");					
				    }
				    readability.cleanStyles( cur );
			}
			cur = sibCur;
		}			
	},
	
	cleanHiddenNode: function (e) {
	    e = e || document;
	    var cur = e.firstChild;
		if(!e)
			return;
	    while ( cur != null ) {
	    	var sibCur = cur.nextSibling;
			if ( cur.nodeType == 1 ) {
				if(cur.getAttribute("style")!= null && (cur.getAttribute("style").indexOf("display:none")!=-1 || cur.getAttribute("style").indexOf("visible:hidden")!=-1)){
	    			cur.parentNode.removeChild(cur);
	    		} else {
	    			readability.cleanHiddenNode(cur);
	    		}
			}
			cur = sibCur;
		}			
	},
	
	/**
     * e中a标签文本占总文本的百分比
     * @param Element
     * @return number (float)
    **/
	getLinkDensity: function (e) {
		var links      = e.getElementsByTagName("a");
		var textLength = readability.getInnerText(e).length;
		var linkLength = 0;
		for(var i=0, il=links.length; i<il;i++)
		{
			linkLength += readability.getInnerText(links[i]).length;
		}		

		return linkLength / textLength;
	},
	
	/**
     * Get an elements class/id weight. 
     * Uses regular expressions to tell if this element looks good or bad.
     *
     * @param Element
     * @return number (Integer)
    **/
	getClassWeight: function (e) {
		var weight = 0;
		 /* Look for a special classname */
		if (typeof(e.className) === 'string' && e.className !== '')
		{
			if(e.className.search(readability.regexps.negativeRe) !== -1)
				weight -= 25;

			if(e.className.search(readability.regexps.positiveRe) !== -1)
				weight += 25;				
		}
		/* Look for a special ID */
		if (typeof(e.id) == 'string' && e.id != '')
		{
			if(e.id.search(readability.regexps.negativeRe) !== -1)
				weight -= 25;

			if(e.id.search(readability.regexps.positiveRe) !== -1)
				weight += 25;				
		}

		return weight;
	},
	
	/**
     * Remove extraneous break tags from a node.
     *
     * @param Element
     * @return void
     **/
	killBreaks: function (e) {
		try {
			//e.innerHTML = e.innerHTML.replace(readability.regexps.killBreaksRe,'<br />');		
		}
		catch (e) {
			dbg("line517"+"KillBreaks failed - this is an IE bug. Ignoring:\n" + e);
		}
	},

	/**
     * Clean a node of all elements of type "tag".
     * (Unless it's a youtube/vimeo video. People love movies.)
     *
     * @param Element
     * @param string tag to clean
     * @return void
     **/
	clean: function (e, tag) {
		var targetList = e.getElementsByTagName( tag );
		//var isEmbed    = (tag == 'object' || tag == 'embed');

		for (var y=targetList.length-1; y >= 0; y--) {
			/* Allow youtube and vimeo videos through as people usually want to see those. */
			//if(isEmbed && targetList[y].innerHTML.search(readability.regexps.videoRe) !== -1)
			//{
			//	continue;
			//}

			targetList[y].parentNode.removeChild(targetList[y]);
		}
	},
	
	/**
     * Clean an element of all tags of type "tag" if they look fishy.
     * "Fishy" is an algorithm based on content length, classnames, link density, number of images & embeds, etc.
     *
     * @return void
     **/
	cleanConditionally: function (e, tag) {
		var tagsList      = e.getElementsByTagName(tag);
		var curTagsLength = tagsList.length;

		/**
         * Gather counts for other typical elements embedded within.
         * Traverse backwards so we can remove nodes at the same time without effecting the traversal.
         *
         * TODO: Consider taking into account original contentScore here.
        **/
		for (var i=curTagsLength-1; i >= 0; i--) {
			var weight = readability.getClassWeight(tagsList[i]);
			var contentScore = (typeof tagsList[i].readability !== 'undefined') ? tagsList[i].readability.contentScore : 0;
			//dbg("Cleaning Conditionally " + tagsList[i] + " (" + tagsList[i].className + ":" + tagsList[i].id + ")" + ((typeof tagsList[i].readability != 'undefined') ? (" with score " + tagsList[i].readability.contentScore) : ''));
            //dbg("weight: " +weight+"  contentScore: "+contentScore); 
            //dbg("标签内容: " + tagsList[i].innerHTML); 
            
            if(weight+contentScore < 0)   
            {
            	//dbg("************删除节点 A ："+ tagsList[i]+"\n"+tagsList[i].innerHTML);
                tagsList[i].parentNode.removeChild(tagsList[i]);
            } 
			else if ( readability.getCharCount(tagsList[i],',')+readability.getCharCount(tagsList[i],'，') < 10) {				 
				/**
                 * If there are not very many commas, and the number of
                 * non-paragraph elements is more than paragraphs or other ominous signs, remove the element.
                **/
				var p      = tagsList[i].getElementsByTagName("p").length;
				var img    = tagsList[i].getElementsByTagName("img").length;
				var li     = tagsList[i].getElementsByTagName("li").length-100;
				var input  = tagsList[i].getElementsByTagName("input").length;

				//var embedCount = 0;
				//var embeds     = tagsList[i].getElementsByTagName("embed");
				//for(var ei=0,il=embeds.length; ei < il; ei++) {
				//	if (embeds[ei].src.search(readability.regexps.videoRe) == -1) {
				//	  embedCount++;	
				//	}
				//}

				var linkDensity   = readability.getLinkDensity(tagsList[i]);
				var contentLength = readability.getInnerText(tagsList[i]).length;
				var toRemove      = false;

				//修复DTS2012121301799
//				if ( img > p ) {
//				 	toRemove = true;
//				} else if(li > p && tag != "ul" && tag != "ol") {
				if(li > p && tag != "ul" && tag != "ol") {
				 	toRemove = true;
				} else if( input > Math.floor(p/3) ) {
				  	toRemove = true; 
				} else if(contentLength < 25 && (img == 0 || img > 2) ) {
					toRemove = true;
				} else if(weight < 25 && linkDensity > .2) {
					toRemove = true;
				} else if(weight >= 25 && linkDensity > .5) {
					toRemove = true;
				//} else if((embedCount == 1 && contentLength < 75) || embedCount > 1) {
				//	toRemove = true;
				}

				if(toRemove) {
					//dbg("************删除节点 B ："+ tagsList[i]+"\n"+tagsList[i].innerHTML);
					tagsList[i].parentNode.removeChild(tagsList[i]);
				}
			}
		}
		 
	},
	
	/**
     * 删除大部分为<a>的p、div
     *
     * @param Element
     * @return void
    **/
	cleanMostATag: function (e, tag) {
		var tagsList      = e.getElementsByTagName(tag);
		var tagsLength = tagsList.length; 
		for (var i=tagsLength-1; i >= 0; i--) {
			if(readability.getLinkDensity(tagsList[i])>=0.5){
				//dbg("cleanMostATag************删除节点："+ tagsList[i].tagName+"\n"+tagsList[i].innerHTML);
				tagsList[i].parentNode.removeChild(tagsList[i]);
			}
		} 
	},

 	/**
     * Clean out spurious headers from an Element. Checks things like classnames and link density.
     *
     * @param Element
     * @return void
    **/
	cleanHeaders: function (e) {
		for (var headerIndex = 1; headerIndex < 7; headerIndex++) {
			var headers = e.getElementsByTagName('h' + headerIndex);
			for (var i=headers.length-1; i >=0; i--) {
				if (readability.getClassWeight(headers[i]) < 0 || readability.getLinkDensity(headers[i]) > 0.33) {
					headers[i].parentNode.removeChild(headers[i]);
				}
			}
		}
	},

	/**
     * Close the email popup. This is a hacktackular way to check if we're in a "close loop".
     * Since we don't have crossdomain access to the frame, we can only know when it has
     * loaded again. If it's loaded over 3 times, we know to close the frame.
     *
     * @return void
     **/
	removeFrame: function () {
	    readability.iframeLoads++;
	    if (readability.iframeLoads > 3)
	    {
	        var emailContainer = document.getElementById('email-container');
	        if (null !== emailContainer) {
	            emailContainer.parentNode.removeChild(emailContainer);
	        }

	        var kindleContainer = document.getElementById('kindle-container');
	        if (null !== kindleContainer) {
	            kindleContainer.parentNode.removeChild(kindleContainer);
	        }

	        readability.iframeLoads = 0;
	    }			
	},
	
	htmlspecialchars: function (s) {
		if (typeof(s) == "string") {
			s = s.replace(/&/g, "&amp;");
			s = s.replace(/"/g, "&quot;");
			s = s.replace(/'/g, "&#039;");
			s = s.replace(/</g, "&lt;");
			s = s.replace(/>/g, "&gt;");
		}
	
		return s;
	}
	
};
 
/**
 * tab的标识
 *
 **/
function saveTabPosition(tabPosition) {
    //dbg("saveTabPosition：" + tabPosition);
	readability.tabPosition = tabPosition;
}

/**
 * 运行js
 *
 **/
function runJS() {
	//dbg("=============运行reader.js==============\n");
    readability.init();
}



